@@ -1,6 +1,6 @@
 from .app import Application  # noqa
+from .app import BatchConsumer  # noqa
 from .app import Router  # noqa
 from .app import run  # noqa
 from .app import run_app  # noqa
 from .app import Subscription  # noqa
-from .app import SubscriptionConsumer  # noqa
