@@ -94,6 +94,11 @@ _aiokafka_consumer_settings = (
     "consumer_timeout_ms",
     "max_poll_records",
     "connections_max_idle_ms",
+    "ssl_context",
+    "security_protocol",
+    "sasl_mechanism",
+    "sasl_plain_username",
+    "sasl_plain_password",
 )
 _aiokafka_producer_settings = (
     "metadata_max_age_ms",
@@ -102,6 +107,11 @@ _aiokafka_producer_settings = (
     "max_request_size",
     "send_backoff_ms",
     "retry_backoff_ms",
+    "ssl_context",
+    "security_protocol",
+    "sasl_mechanism",
+    "sasl_plain_username",
+    "sasl_plain_password",
 )
 
 
@@ -133,6 +143,64 @@ class Router:
 
         self._event_handlers[name].append(handler)
 
+    def _subscribe(
+        self,
+        group: str,
+        *,
+        consumer_id: str = None,
+        pattern: str = None,
+        topics: List[str] = None,
+        timeout_seconds: float = None,
+        concurrency: int = None,
+    ) -> Callable:
+        def inner(func: Callable) -> Callable:
+            # If there is no consumer_id use the group instead
+            subscription = Subscription(
+                consumer_id or group,
+                func,
+                group or func.__name__,
+                pattern=pattern,
+                topics=topics,
+                concurrency=concurrency,
+                timeout_seconds=timeout_seconds,
+            )
+            self._subscriptions.append(subscription)
+            return func
+
+        return inner
+
+    def subscribe_to_topics(
+        self,
+        topics: List[str],
+        group: str,
+        *,
+        timeout_seconds: float = None,
+        concurrency: int = None,
+    ) -> Callable:
+        return self._subscribe(
+            group=group,
+            topics=topics,
+            pattern=None,
+            timeout_seconds=timeout_seconds,
+            concurrency=concurrency,
+        )
+
+    def subscribe_to_pattern(
+        self,
+        pattern: str,
+        group: str,
+        *,
+        timeout_seconds: float = None,
+        concurrency: int = None,
+    ) -> Callable:
+        return self._subscribe(
+            group=group,
+            topics=None,
+            pattern=pattern,
+            timeout_seconds=timeout_seconds,
+            concurrency=concurrency,
+        )
+
     def subscribe(
         self,
         stream_id: str,
@@ -141,18 +209,14 @@ class Router:
         timeout_seconds: float = None,
         concurrency: int = None,
     ) -> Callable:
-        def inner(func: Callable) -> Callable:
-            subscription = Subscription(
-                stream_id,
-                func,
-                group or func.__name__,
-                concurrency=concurrency,
-                timeout_seconds=timeout_seconds,
-            )
-            self._subscriptions.append(subscription)
-            return func
-
-        return inner
+        """Keep backwards compatibility"""
+        return self._subscribe(
+            group=group,
+            topics=None,
+            pattern=stream_id,
+            timeout_seconds=timeout_seconds,
+            concurrency=concurrency,
+        )
 
     def schema(
         self,
@@ -244,6 +308,11 @@ class Application(Router):
                 self._topic_prefix,
                 replication_factor=self._replication_factor,
                 kafka_api_version=self._kafka_api_version,
+                ssl_context=self.kafka_settings.get("ssl_context"),
+                security_protocol=self.kafka_settings.get("security_protocol", "PLAINTEXT"),
+                sasl_mechanism=self.kafka_settings.get("sasl_mechanism"),
+                sasl_plain_username=self.kafka_settings.get("sasl_plain_username"),
+                sasl_plain_password=self.kafka_settings.get("sasl_plain_password"),
             )
         return self._topic_mng
 
@@ -335,6 +404,14 @@ class Application(Router):
 
         if not headers:
             headers = []
+        else:
+            # this is just to check the headers shape
+            try:
+                for _, _ in headers:
+                    pass
+            except ValueError:
+                # We want to be resilient to malformated headers
+                logger.exception(f"Malformed headers: '{headers}'")
 
         if isinstance(tracer.scope_manager, ContextVarsScopeManager):
             # This only makes sense if the context manager is asyncio aware
@@ -345,6 +422,7 @@ class Application(Router):
                     format=opentracing.Format.TEXT_MAP,
                     carrier=carrier,
                 )
+
                 header_keys = [k for k, _ in headers]
                 for k, v in carrier.items():
                     # Dont overwrite if they are already present!
@@ -466,7 +544,6 @@ class Application(Router):
 
     async def consume_for(self, num_messages: int, *, seconds: Optional[int] = None) -> int:
         consumed = 0
-
         self._subscription_consumers = []
         tasks = []
         for subscription in self._subscriptions:
@@ -478,13 +555,10 @@ class Application(Router):
                     raise StopConsumer
 
             consumer = BatchConsumer(
-                stream_id=subscription.stream_id,
-                group_id=subscription.group,
-                coro=subscription.func,
+                subscription=subscription,
                 app=self,
                 event_handlers={"message": [on_message]},
-                concurrency=subscription.concurrency or 1,
-                timeout_seconds=subscription.timeout,
+                auto_commit=self.auto_commit,
             )
 
             self._subscription_consumers.append(consumer)
@@ -512,12 +586,9 @@ class Application(Router):
 
         for subscription in self._subscriptions:
             consumer = BatchConsumer(
-                stream_id=subscription.stream_id,
-                group_id=subscription.group,
-                coro=subscription.func,
+                subscription=subscription,
                 app=self,
-                concurrency=subscription.concurrency or 1,
-                timeout_seconds=subscription.timeout,
+                auto_commit=self.auto_commit,
             )
             self._subscription_consumers.append(consumer)
 
@@ -570,9 +641,16 @@ async def run_app(app: Application) -> None:
         fut = asyncio.create_task(app.consume_forever())
         for signame in {"SIGINT", "SIGTERM"}:
             loop.add_signal_handler(getattr(signal, signame), partial(_sig_handler, app))
-        await fut
+        done, pending = await fut
         logger.debug("Exiting consumer")
 
+        await app.stop()
+        # re-raise any errors so we can validate during tests
+        for task in done:
+            exc = task.exception()
+            if exc is not None:
+                raise exc
+
 
 def run(app: Optional[Application] = None) -> None:
     if app is None:
