@@ -4,7 +4,6 @@ import aiokafka
 
 if TYPE_CHECKING:  # pragma: no cover
     from .app import SchemaRegistration
-    from .app import SubscriptionConsumer
 else:
     SchemaRegistration = SubscriptionConsumer = None
 
@@ -33,9 +32,13 @@ class StopConsumer(Exception):
     ...
 
 
+class HandlerTaskCancelled(Exception):
+    def __init__(self, record: aiokafka.ConsumerRecord):
+        self.record = record
+
+
 class ConsumerUnhealthyException(Exception):
-    def __init__(self, subscriber_consumer: SubscriptionConsumer, reason: str):
-        self.subscriber_consumer = subscriber_consumer
+    def __init__(self, reason: str):
         self.reason = reason
 
 
